@@ -8,6 +8,17 @@ import errors
 
 
 class Connection(object):
+
+    Error = errors.Error
+    Warning = errors.Warning
+    InterfaceError = errors.InterfaceError
+    DatabaseError = errors.DatabaseError
+    InternalError = errors.InternalError
+    OperationalError = errors.OperationalError
+    ProgrammingError = errors.ProgrammingError
+    IntegrityError = errors.IntegrityError
+    DataError = errors.DataError
+    NotSupportedError = errors.NotSupportedError
     
     def __init__(self, *args, **kwargs):
         """There are 2 ways of instantiation:
@@ -18,7 +29,7 @@ class Connection(object):
         http://www.postgresql.org/docs/9.1/static/libpq-connect.html
         """
         connstr = self.build_connstr(*args, **kwargs)
-        self._db = libpq.c.PQconnectdb(connstr)
+        self._db = libpq.PQconnectdb(connstr)
         self._closed = False
         if self._get_status() != CONNECTION_OK:
             raise errors.DatabaseError(self._get_error_msg())
@@ -40,7 +51,7 @@ class Connection(object):
     def _get_status(self):
         if self._closed:
             return
-        return libpq.c.PQstatus(self._db)
+        return libpq.PQstatus(self._db)
 
     def _get_status_string(self):
         if self._closed:
@@ -48,14 +59,14 @@ class Connection(object):
         return CONNECTION_STATUSES[self._get_status()]
 
     def _get_error_msg(self):
-        return ctypes.string_at(libpq.c.PQerrorMessage(self._db))
+        return libpq.PQerrorMessage(self._db)
 
     def _check_closed(self):
         if self._closed:
             raise errors.InterfaceError("Connection already closed")
 
     def _get_transaction_status(self):
-        return libpq.c.PQtransactionStatus(self._db)
+        return libpq.PQtransactionStatus(self._db)
 
     def cursor(self):
         self._check_closed()
@@ -70,23 +81,23 @@ class Connection(object):
         self._check_closed()
         info = {}
         info['server_version'] = self._get_server_version()
-        info['db'] = ctypes.string_at(libpq.c.PQdb(self._db))
-        info['user'] = ctypes.string_at(libpq.c.PQuser(self._db))
+        info['db'] = libpq.PQdb(self._db)
+        info['user'] = libpq.PQuser(self._db)
         # This raises a segfault somehow
-        # info['host'] = ctypes.string_at(libpq.c.PQhost(self._db))
-        info['port'] = ctypes.string_at(libpq.c.PQport(self._db))
+        info['host'] = libpq.c.PQhost(self._db)
+        info['port'] = libpq.PQport(self._db)
         return info
 
     def _get_server_version(self):
-        version = libpq.c.PQserverVersion(self._db)
+        version = libpq.PQserverVersion(self._db)
         # TODO: parse the number according to the spec
         return version
 
     def get_client_encoding(self):
-        return ctypes.string_at(libpq.c.pg_encoding_to_char(libpq.c.PQclientEncoding(self._db)))
+        return libpq.pg_encoding_to_char(libpq.PQclientEncoding(self._db))
 
     def set_client_encoding(self, enc):
-        result = libpq.c.PQsetClientEncoding(self._db, enc)
+        result = libpq.PQsetClientEncoding(self._db, enc)
         if result:
             raise errors.ProgrammingError("Cannot set encoding to '%s'" % enc)
         self._encoding = self.get_client_encoding()
@@ -95,16 +106,18 @@ class Connection(object):
         warnings.warn('Ignoring "set_isolation_level"')
 
     def commit(self):
+        self._check_closed()
         if self._get_transaction_status() in (PQTRANS_INTRANS, PQTRANS_INERROR):
             cursor = self.cursor()
             cursor.execute('COMMIT')
 
     def rollback(self):
+        self._check_closed()
         if self._get_transaction_status() in (PQTRANS_INTRANS, PQTRANS_INERROR):
             cursor = self.cursor()
             cursor.execute('ROLLBACK')
 
     def close(self):
         self._check_closed()
-        libpq.c.PQfinish(self._db)
+        libpq.PQfinish(self._db)
         self._closed = True
\ No newline at end of file
