@@ -44,14 +44,14 @@ class Cursor(object):
             for index, name in enumerate(self._field_names):
                 data.append((
                     name, # name
-                    self._field_oids[index], # type_code
+                    self._field_types[index], # type_code
                     None, # display_size
                     None, # internal_size
                     None, # precision
                     None, # scale
                     None, # null_ok
                 ))
-        return data
+            return data
 
     @property
     def field_names(self):
@@ -79,11 +79,12 @@ class Cursor(object):
     def callproc(self, procname, params=()):
         """Call a stored database procedure with the given name.
         """
-        return self.execute('select * from %s(%s)' % (
+        self.execute('select * from %s(%s)' % (
             procname, ', '.join('%s' for i in params)
         ), params)
+        return params
 
-    def execute(self, query, args=None):
+    def execute(self, query, args=()):
         """
         Prepare and execute a database operation (query or
         command).  Parameters may be provided as sequence or
@@ -95,9 +96,30 @@ class Cursor(object):
         self._check_closed()
         self._clear()
         self._init()
+        self._check_transaction()
+        query, replacements = self._process_query(query)
+        return self._execute(query, replacements, args)
+
+    def _check_transaction(self):
         if self._connection._get_transaction_status() == PQTRANS_IDLE:
             self._execute('BEGIN')
-        return self._execute(query, args)
+
+    def executemany(self, query, args=()):
+        self._connection._check_closed()
+        self._check_closed()
+        self._clear()
+        self._init()
+        self._check_transaction()
+        query, replacements = self._process_query(query)
+        
+        rowcount = 0
+        
+        for arg_group in args:
+            self._execute(query, replacements, arg_group)
+            # Sum the rowcounts if every one of them is countable
+            if rowcount >= 0:
+                rowcount += self.rowcount
+        self._rowcount = rowcount
 
     def __iter__(self):
         while True:
@@ -110,32 +132,45 @@ class Cursor(object):
         if self._closed:
             raise errors.ProgrammingError('The cursor is already closed')
 
-    def _execute(self, query, args=None):
-        """Execute the query through libpq execution api"""
+    def _process_query(self, query):
+        """Replace "%s" arguments with libpq $1,$2 style"""
+
         if isinstance(query, unicode):
             query = query.encode(self._connection._encoding)
+
+        query = query.replace('%%', '%\0%')
+        replacements = 0
+        for i in xrange(query.count('%s')):
+            query = query.replace('%s', '$%s' % (replacements + 1), 1)
+            replacements += 1
+        query = query.replace('%\0%', '%%')
+        return query, replacements
+
+    def _execute(self, query, replacements=0, args=()):
+        """Execute the query through libpq execution api"""
         if not args:
-            self._result = libpq.c.PQexec(self.connection._db, query)
+            # We can use the simple "PQexec" function if we do not have
+            # any arguments
+            self._result = libpq.PQexec(self.connection._db, query)
         else:
+            n_args = len(args)
+
             # Prepare the arguments for passing to PQexecParams
             # convert them to postgres types according to registered
-            # datatypes see datatypes.py for details
-            n_args = len(args)
-            if query.count('%s') != n_args:
+            # datatypes, see datatypes.py for details
+
+            if replacements != len(args):
                 raise errors.ProgrammingError('Number of arguments and '
                                               'replacements do not match')
 
-            # Replace "%s" arguments with libpq $1,$2 style
-            for i in xrange(n_args):
-                query = query.replace('%s', '$%s' % (i + 1), 1)
-
             # Create c types for parameter arrays
             arr_chars_t = ctypes.c_char_p * n_args
             arr_ints_t = ctypes.c_int * n_args
+            arr_uints_t = ctypes.c_uint * n_args
 
             def process_arg(arg):
                 """Convert the python argument for PQexecParams"""
-                
+
                 # TODO: fix a bug (there is no such issue in psycopg2)
                 # cur.execute('select %s', [None])
                 # DatabaseError: ERROR: could not determine data type
@@ -146,11 +181,11 @@ class Cursor(object):
                 return oid, value, len(value), 0
 
             types, values, lengths, formats = zip(*map(process_arg, args))
-            self._result = libpq.c.PQexecParams(
+            self._result = libpq.PQexecParams(
                 self.connection._db, query,
                 n_args, # int nParams,
                 # Passing oids only creates problems, so ignore that param
-                # arr_ints_t(*types), # const Oid *paramTypes,
+                # arr_uints_t(*types), # const Oid *paramTypes,
                 None,
                 arr_chars_t(*values), # const char * const *paramValues,
                 arr_ints_t(*lengths), # const int *paramLengths,
@@ -159,36 +194,51 @@ class Cursor(object):
             )
 
         self._query = query
+        self._process_result()
         
+    def _process_result(self):
         # The query has been executed, now process the result
-        status = libpq.c.PQresultStatus(self._result)
-        if status == PGRES_FATAL_ERROR:
+        self._status = libpq.PQresultStatus(self._result)
+        if self._status == PGRES_FATAL_ERROR:
             exc = errors.DatabaseError(self._get_error_msg())
             self._clear()
             raise exc
-        elif status == PGRES_COMMAND_OK:
-            self._cleared = False
-            self._rowcount = int(ctypes.string_at(libpq.c.PQcmdTuples(self._result)) or -1)
-            self._rownumber = self._rowcount
+        elif self._status == PGRES_COMMAND_OK:
+            self._rowcount = int(libpq.PQcmdTuples(self._result) or -1)
             self._clear()
-        elif status == PGRES_TUPLES_OK:
-            self._rowcount = libpq.c.PQntuples(self._result)
+        elif self._status == PGRES_TUPLES_OK:
+            self._rowcount = libpq.PQntuples(self._result)
             self._rownumber = 0
             self._field_names, self._field_types, self._field_oids = zip(*self._get_fields())
         else:
-            # AFAIK, this should not happen
-            raise NotImplementedError
+            raise errors.Error('Unexpected pgresultstatus %s' % self._status)
+
+    def setinputsizes(self, sizes):
+        """DBAPI 2.0 setinputsizes
+
+        Implementations are free to have this method do nothing
+        and users are free to not use it.
+        """
+        pass
+
+    def setoutputsize(self, size, column=None):
+        """DBAPI 2.0 setoutputsize
+
+        Implementations are free to have this method do nothing
+        and users are free to not use it.
+        """
+        pass
 
     def _get_fields(self):
         """Get a list of (field_name, field_type) tuples from the result"""
         if not self._result:
             raise errors.ProgrammingError('This method should be called'
                                           ' when cursor._result is available')
-        nfields = libpq.c.PQnfields(self._result)
+        nfields = libpq.PQnfields(self._result)
         fields = []
         for i in xrange(nfields):
-            fname = ctypes.string_at(libpq.c.PQfname(self._result, i))
-            foid = libpq.c.PQftype(self._result, i)
+            fname = libpq.PQfname(self._result, i)
+            foid = libpq.PQftype(self._result, i)
             ftype = datatypes.get_type_by_oid(foid)
             fields.append((fname, ftype, foid))
         return tuple(fields)
@@ -197,22 +247,18 @@ class Cursor(object):
         """Get an error message describing why failure occured"""
         if not self._result:
             return
-        return ctypes.string_at(libpq.c.PQresultErrorMessage(self._result))
-
-    def executemany(self, query, args):
-        for arg_group in args:
-            self.execute(query, arg_group)
+        return libpq.PQresultErrorMessage(self._result)
 
     def fetchone(self):
-        if self._rowcount == -1:
+        if self._rowcount == -1 or self._status == PGRES_COMMAND_OK:
             raise errors.ProgrammingError('Nothing to fetch')
         if self._rowcount == self._rownumber:
             return
         data = []
         for i, type in enumerate(self._field_types):
-            value = ctypes.string_at(libpq.c.PQgetvalue(self._result, self._rownumber, i))
+            value = libpq.PQgetvalue(self._result, self._rownumber, i)
             if not value:
-                is_null = libpq.c.PQgetisnull(self._result, self._rownumber, i)
+                is_null = libpq.PQgetisnull(self._result, self._rownumber, i)
                 if is_null:
                     data.append(None)
                     continue
@@ -240,11 +286,12 @@ class Cursor(object):
         self._field_oids = None
         self._closed = False
         self._query = None
+        self._status = None
 
     def _clear(self):
         """Clear the result of previous execution."""
         if self._result and not self._cleared:
-            libpq.c.PQclear(self._result)
+            libpq.PQclear(self._result)
             self._cleared = True
             
     def __del__(self):
