@@ -5,7 +5,7 @@ def Date(year,month,day):
     return datetime.date(year, month, day)
 
 def Time(hour,minute,second):
-    raise NotImplementedError
+    return datetime.time(hour, minute, second)
 
 def Timestamp(year,month,day,hour,minute,second):
     return datetime.datetime(year, month, day, hour, minute, second)
@@ -22,8 +22,10 @@ def TimestampFromTicks(ticks):
 def Binary(str):
     return str
 
-STRING = str
-BINARY = STRING
-NUMBER = int
-DATETIME = datetime.datetime
-ROWID = int
\ No newline at end of file
+import datatypes
+
+STRING = datatypes.Unicode
+BINARY = datatypes.String
+NUMBER = datatypes.Integer
+DATETIME = datatypes.DateTime
+ROWID = datatypes.ROWID
\ No newline at end of file
